/*
---------------------------------------------------------------------------
 Algo11-1.c   C program for implementing Algorithm 11.1
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 11.1 (Power Method).
 Section	11.2, The Power Method, Page 557
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 11.1 (Power Method)


 To compute the dominant value  Lambda_1  and its associated
 eigenvector  V_1  for the  n x n  matrix  A. It is assumed 
 that the  n  eigenvalues have the dominance property

 |Lambda_1| > |Lambda_2| >= |Lambda_3| >= ... >= |Lambda_n| > 0

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define MaxOrder 50

#define MAX(a,b) a > b ? a : b

/* -------------------------------------------------------- */

/*  Main program for algorithm 11.1  */

void main(void)

{
    int i, j;                  /* Loop counter                   */
    int N;                     /* Order of Matrix                */
    double Epsilon = 1E-7;     /* Tolerance                      */
    double Max = 100;          /* Maximum number of iterations   */
    double X[MaxOrder], Y[MaxOrder];
    double A[MaxOrder][MaxOrder]; /* Matrix                      */
    double C1, DC, DV, Lambda = 0;
    int Count = 0, Iterating = 1;
    double Sum, Err = 1;
    double MaxElement;


    printf("--------------Power Method---------------\n");
    printf("--------- Example 11.5 on page 550 -------------\n");
    printf("-----------------------------------------------\n");

    printf("Please enter order of Matrix  A ( < %d !)\n", MaxOrder + 1);
    scanf("%d",&N);
    if( N > MaxOrder )
    {
    printf(" Number of steps must be less than %d\n",MaxOrder+1);
    printf(" Terminating. Sorry\n");
    exit(0);
    }
    printf("Please enter elements of matrix  A  row by row:\n");

    for ( i = 0; i < N; i++)
    {
        for ( j = 0; j < N; j++)
        {
            printf(" Enter Element No. %d of row %d\n", j+1, i+1);
            scanf("%lf", &A[i][j]);
            printf("You entered A[%d][%d]= %lf\n", i+1, j+1, A[i][j] );
            printf("------------------------------------------------\n");
        }
    }

    printf("\n");

    /* Initialize vector X */

    for ( i = 0; i < N; i++)   X[i] = 1.0;

    while( (Count <= Max) && (Iterating == 1) )
    {

    /* Perform Matrix-Vector multiplication */

    for ( i = 0; i < N; i++ )
    {
        Y[i] = 0;
        for ( j = 0; j < N; j++ ) Y[i] += A[i][j] * X[j];
    }

    /* Find largest element of vector Y */
    /* Do what function MaxElement(X,N) in the book does */

    MaxElement = 0;
    for ( j = 0; j < N; j++ ) 
    {
        if( fabs(Y[j]) > fabs(MaxElement) ) MaxElement = Y[j];
    }

    C1 = MaxElement;

    DC = fabs(Lambda - C1);

    for ( i = 0; i < N; i++) Y[i] *= 1.0/C1;

    /* Do what function DIST(X,Y,N) in the book does */

    Sum = 0;
    for ( i = 0; i < N; i++) Sum += pow( ( Y[i] - X[i] ), 2.0);
    DV  = sqrt(Sum);
    Err = MAX(DC,DV);

    /* Update vector X and scalar Lambda */

    for ( i = 0; i < N; i++) X[i] = Y[i];
    Lambda = C1;

    Iterating = 0;

    if( Err > Epsilon) Iterating = 1;

    Count++;

    } /* End of while loop */

    
    /* Output vector X and scalar Lambda */

    printf("---------------------------------------\n");

    for ( j = 0; j < N; j++) printf("X[%d] = %lf\n", j, X[j]);
 
    printf("---------------------------------------\n");
    printf("Lambda = %lf\n", Lambda);


}   /* End of main program */

