/*
---------------------------------------------------------------------------
 Algo11-2.c   C program for implementing Algorithm 11.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 11.2 (Shifted Inverse Power Method).
 Section   11.2, The Power Method, Page 558
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 11.2 (Shifted Inverse Power Method)


 To compute the dominant value Lambda_j and its associated
 eigenvector  V_j  for the  n x n  matrix  A. It is assumed 
 that the  n  eigenvalues are  Lambda_1 < Lambda_2 < ... < Lambda_n
 and that  alpha  is a real number such that 

 |Lambda_j - alpha| < |Lambda_i - alpha|  for each 

                                          i = 1,2,..., j-1, j+1,..., n.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define Limit  20     /* for subroutine Algo32 */

#define MaxOrder 30

#define MAX(a,b) a > b ? a : b

/* -------------------------------------------------------- */

/*  Main program for algorithm 11.2  */

void main(void)

{
    extern double Algo32();

    int i, j;                  /* Loop counter                   */
    int N;                     /* Order of Matrix                */
    double Epsilon = 1E-7;     /* Tolerance                      */
    double Max = 100;          /* Maximum number of iterations   */
    double X[MaxOrder], Y[MaxOrder];
    double A[MaxOrder][MaxOrder]; /* Matrix                      */
    double A1[MaxOrder][MaxOrder], B[MaxOrder][MaxOrder+1];
    double C1, DC, DV, Lambda = 0;
    int Count = 0, Iterating = 1;
    double Sum, Err = 1;
    double MaxElement;
    double alpha;              /* shift parameter                */ 


    printf("--------------Power Method---------------------\n");
    printf("--------- Example 11.6 on page 555 ------------\n");
    printf("-----------------------------------------------\n");

    printf("Please enter shifting parameter Alpha:\n");
    printf("See example in book on page 555/556.\n");
    scanf("%lf", &alpha);
    printf("-----------------------------------------------\n");
    printf("\n");

    printf("Please enter order of Matrix  A ( < %d !)\n", MaxOrder + 1);
    scanf("%d",&N);
    if( N > MaxOrder )
    {
    printf(" Order of matrix must be less than %d\n",MaxOrder+1);
    printf(" Terminating. Sorry\n");
    exit(0);
    }
    printf("Please enter elements of matrix  A  row by row:\n");

    for ( i = 0; i < N; i++)
    {
        for ( j = 0; j < N; j++)
        {
            printf(" Enter Element No. %d of row %d\n", j+1, i+1);
            scanf("%lf", &A[i][j]);
            printf("You entered A[%d][%d]= %lf\n", i+1, j+1, A[i][j] );
            printf("------------------------------------------------\n");
        }
    }

    printf("\n");

    /* Initialize vector X */

    for ( i = 0; i < N; i++)   X[i] = 1.0;

    /* build A1 matrix */
    
    for ( i = 0; i < N; i++ )
    {
       for ( j = 0; j < N; j++ )
       {
          A1[i][j] = A[i][j];
          if( i == j) A1[i][j] -= alpha;

       }
    }  

    while( (Count <= Max) && (Iterating == 1) )
    {

    /* Solve a linear system ( A1 Y = X ) */

    /* Build the augmented matrix AX */

    for ( i = 0; i < N; i++ )
    {
       for ( j = 0; j <= N; j++ )
       {
          if( j == N) B[i][j] = X[i];
          else B[i][j] = A1[i][j];
       }
    } 

    Algo32(B, Y, N);

    /* Find largest element of vector Y */
    /* Do what function MaxElement(X,N) in the book does */

    MaxElement = 0;
    for ( j = 0; j < N; j++ ) 
    {
        if( fabs(Y[j]) > fabs(MaxElement) ) MaxElement = Y[j];
    }

    C1 = MaxElement;

    DC = fabs(Lambda - C1);

    for ( i = 0; i < N; i++) Y[i] *= 1.0/C1;

    /* Do what function DIST(X,Y,N) in the book does */

    Sum = 0;
    for ( i = 0; i < N; i++) Sum += pow( ( Y[i] - X[i] ), 2.0);
    DV  = sqrt(Sum);
    Err = MAX(DC,DV);

    /* Update vector X and scalar Lambda */

    for ( i = 0; i < N; i++) X[i] = Y[i];
    Lambda = C1;

    Iterating = 0;

    if( Err > Epsilon) Iterating = 1;

    Count++;

    } /* End of while loop */

    
    /* Output vector X and scalar Lambda */

    printf("--------Solution : --------------------\n");

    for ( j = 0; j < N; j++) printf("X[%d] = %lf\n", j, X[j]);
 
    printf("---------------------------------------\n");
    printf("Lambda = %lf\n", Lambda);


}   /* End of main program */

/*=====================================================*/
/*
   From :
   Numerical Methods for Mathematics, Science, and Engineering
   by : John H. Mathews
   Publisher : Prentice-Hall, Inc.
   Page 156
   Translated to  C  by : Norman Fahrer
   1995
*/
/*---------------------------------------------------------
 Algorithm 3.2 ( Upper Triangularization Followed by Back
                 Substitution).

 To construct the solution to AX = B, by first reducing the
 augmented matrix [A,B] to upper-triangular form and the
 performing back substitution.

 ---------------------------------------------------------*/

/*  remember : in C the fields begin with element 0  */


double Algo32(B, Y, N)
int N;
double B[MaxOrder][MaxOrder+1];
double *Y;

{
    int K, P, C, J;                  /* Loop counters               */
    int    Row[MaxOrder];            /* Field with row-number       */
    double SUM, M;                                                       
    int  T;

    /* Initialize the pointer vector */

    for (J = 1; J<= N; J++) Row[J-1] = J - 1;

    /* Start upper-triangularization */

    for (P = 1; P <= N - 1; P++)
    {

    /* Find pivot element */  

      for (K = P + 1; K <= N; K++)
      {
        if ( fabs(B[Row[K-1]][P-1]) > fabs(B[Row[P-1]][P-1]) )
        {
        /* Switch the index for the p-1 th pivot row if necessary */ 
           T      = Row[P-1];
           Row[P-1] = Row[K-1];
           Row[K-1] = T;
        }
      } /* End of simulated row interchange */

        if (B[Row[P-1]][P-1] == 0)
        {   
           printf("The matrix is SINGULAR !\n");
           printf("Cannot use algorithm --> exit\n");
           exit(1);
        }

     /* Form multiplier */

     for (K = P + 1; K <= N; K++)
     { 
       M = B[Row[K-1]][P-1] / B[Row[P-1]][P-1];
      
       /* Eliminate Y_(p-1) */

       for (C = P + 1; C <= N + 1; C++)
       {
          B[Row[K-1]][C-1] -= M * B[Row[P-1]][C-1];
       }
     } 

  } /* End of upper-triangularization routine */


    if( B[Row[N-1]][N-1] == 0)
    {
      printf("The matrix is SINGULAR !\n");
      printf("Cannot use algorithm --- exit\n");
      exit(1);
    }

    /* Start the back substitution */
   
    Y[N-1] = B[Row[N-1]][N] / B[Row[N-1]][N-1];

    for (K = N - 1; K >= 1; K--)
    {
      SUM = 0;
      for (C = K + 1; C <= N; C++)
      {
           SUM += B[Row[K-1]][C-1] * Y[C-1];   
      }

      Y[K-1] = ( B[Row[K-1]][N] - SUM) / B[Row[K-1]][K-1];

    }  /* End of back substitution */


}  /* End of subroutine */

