/*
---------------------------------------------------------------------------
 Algo11-5.c   C program for implementing Algorithm 11.5
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 11.5 (The QL Method with Shifts).
 Section   11.4, Eigenvalues of Symmetric Matrices, Page 587
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 11.5 ( The QL Method with Shifts )

 Given the symmetric tridiagonal matrix A whose diagonal
 elements are stored in  D[1], D[2], ... , D[N]  and whose
 subdiagonal elements are stored in  E[1], E[2], ... , E[N-1],
 to construct the eigenvalues of  A  and store them in 
 D[1], D[2], ..., D[N];

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define MaxOrder 50

/* Prototypes       */

void FindShift(void);
void FormL(void);
void FormA(void);

/* Global variables */

double D[MaxOrder], E[MaxOrder], S[MaxOrder];
double C[MaxOrder], P[MaxOrder], Q[MaxOrder];
double SH, R1, R2;
int N;
int M = 1;

/* -------------------------------------------------------- */

/*  Main program for algorithm 11.5  */

void main(void)

{
    double DELTA = 1E-08;      /* Tolerance                      */             

    double SHIFT = 0;
    int    I, J;               /* Loop counters                  */
    int    K, Cond;

    

    printf("-------------------The QL Method with Shifts-------------------\n");

    printf("---------------Try example 11.9 on page 584/585----------------\n");

    printf("-----( see also algorithm 11.4 to create tridiagonal forms )---\n");

    printf("Please enter order of Matrix  A ( < %d !)\n", MaxOrder);
    scanf("%d",&N);
    if( N > MaxOrder )
    {
    printf(" Order of Matrix must be less than %d\n",MaxOrder);
    printf(" Terminating. Sorry\n");
    exit(0);
    }
    printf("-----------------------------------------------------------\n");
    printf("Enter diagonal elements of the matrix one by one :\n");
    printf("Remember : Matrix is assumed to be symmetric !\n");


    /* In C arrays start with 0 . Here we keep the D[0] and E[0]
       elements as a dummy doing nothing.
       For security we initialize them though. */

    /* dummy fields : not used by algorithm here */

    D[0] = E[0] = 0.0;

    for ( I = 1; I <= N; I++)
    {
        printf(" Enter DIAGONAL Element No. %d\n", I);
        scanf("%lf", &D[I]);
        printf("You entered A[%d][%d]= %lf\n", I, I, D[I] );
    }


    for ( I = 1; I < N; I++)
    {
        printf(" Enter SUB-Diagonal element A[%d][%d] :\n", I, I+1);
        scanf("%lf", &E[I]);
        printf("You entered A[%d][%d]= %lf\n", I, I+1, E[I]);
        printf("------------------------------------------------\n");
    }
    
    /* Begin with while-loop */

    while ( M <= N - 2 )
    {
     K    = 1;
     Cond = 0;

        while ( (K < 50) && (Cond == 0) )
        {
            FindShift();
            if ( fabs(E[M]) > DELTA )
            {
                SHIFT += SH;
                for ( J = M; J <= N; J++) D[J] -= SH;
            }
            else
            {
                D[M] += SHIFT;
                M++;
                Cond = 1;
            }

            FormL();
            FormA();
            K++;
        }
    }   /* end of major while-loop */

    FindShift();

    D[N-1] = R1 + SHIFT;
    D[N]   = R2 + SHIFT;

    for ( J = 1; J <= N - 1; J++ ) E[J] = 0;

    printf("--- Output : Eigen-Values of the tridiagonal matrix :\n");

    for ( J = 1; J<= N; J++) printf("\nD[%d] = %12.7lf  ", J, D[J]);

    printf("\n------------------------------------------------\n");

}   /* End of main program */

/*=============================================================*/

void FindShift(void)
{

    double B1, C1, D1;

    /* Find the shift parameter */

    B1 = - ( D[M+1] + D[M] );
    C1 = D[M+1] * D[M] - E[M] * E[M];
    D1 = sqrt( fabs(B1 * B1 - 4.0 * C1) );

    if ( B1 > 0 )
    {
        R1 = -0.5 * (B1 + D1);
        R2 = -2.0 * C1 /(B1 + D1);
    }
    else
    {
        R1 = ( -B1 + D1 ) / 2.0;
        R2 = 2.0 * C1 / ( -B1 + D1 );
    }

    SH = R1;

    if ( fabs(D[M] - R2) < fabs(D[M] - R1) ) SH = R2;

    return; 
}

/*=============================================================*/

void FormL(void)
{

    int J;
    double PJ1, QJ;

    /* Construct L and Q implicitly. 
       Make copy of vectors D and E */

    for ( J = 1; J <= N; J++ ) P[J] = D[J];
    for ( J = 1; J <= N - 1; J++ ) Q[J] = E[J];

    PJ1 = P[N];       /* PJ1 is old value of P[J+1] */
    QJ  = Q[N-1];     /* QJ  is old value of Q[J]   */
    
    for ( J = N - 1; J >= M; J-- )
    {
        P[J+1] = sqrt( PJ1 * PJ1 + Q[J] * Q[J] );
        C[J]   = PJ1  / P[J+1];
        S[J]   = Q[J] / P[J+1];
        Q[J]   = C[J] * QJ + S[J] * D[J];
        PJ1    = C[J] * D[J] - S[J] * QJ;

        if ( J > M ) QJ = C[J] * Q[J-1];

        /* { R[J-1] = S[J] * Q[J-1] would be dead code */
    }

    P[M] = PJ1;

    return;
}

/*=============================================================*/

void FormA(void)
{

    int J;
    int I;

    /* Construct the product LQ */

    D[N]    = S[N-1] * Q[N-1] + C[N-1] * P[N];
    E[N-1]  = S[N-1] * P[N-1];

    for ( J = N - 2; J >= M; J-- )
    {
        D[J+1] = S[J] * Q[J] + C[J] * C[J+1] * P[J+1];
        E[J]   = S[J] * P[J];
    }

    D[M] = C[M] * P[M];

    return;
}

