/*
---------------------------------------------------------------------------
 Algo2-8.c   C program for implementing Algorithm 2.8
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 2.8 (Muller's Method).
 Section   2.5, Aitken's Process & Steffensen's & Muller's Methods, Page 97
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 2.8 (Muller's Method). To find a root of the
 equation  f(x) = 0  given three distinct initial approximations
 p_0, p_1 and p_2. (Eliminate the step that suppresses
 complex roots if complex variables are NOT used).

---------------------------------------------------------------------------
*/

/* User has to supply a function named :  ffunction

   An example is included in this program     */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction" : from Sec 2.5, p. 100 */

    double ffunction(double x)

    {
         return ( pow(x,3) - x - 2 );
    }


/* -------------------------------------------------------- */

/*  Main program for algorithm 2.8  */

    void main()

{
    double Delta   = 1E-6;        /* Tolerance  */
    double Epsilon = 1E-6;        /* Tolerance  */
    double Small   = 1E-6;        /* Tolerance  */

    int Max = 99;        /* Maximum number of iterations  */
    int Satisfied = 0;   /* Condition fo loop termination */
    int K;               /* Counter for loop              */

    double Y0, Y1, Y2;

    double P0, P1, P2;  /* INPUT : Initial approximation */
    double P3;
    double H0, H1;    /* Differences        */
    double E0;        /* Difference         */
    double E1;        /* Difference         */
    double Det;       /* Determinants       */
    double Disc;      /* Discriminant       */
    double A, B, C, U, V, Z;
    double RelErr;    /* Relative Error     */


    printf("-----------------------------------------------------\n");
    printf("Please enter THREE different initial approximations !\n");
    printf("EXAMPLE: 1.0  1.2  1.4 !\n");
    scanf("%lf %lf %lf",&P0, &P1, &P2);
    printf("-----------------------------------------------------\n");
    printf("The initial approximation are : %lf\n",P0);
    printf("                              : %lf\n",P1);
    printf("                              : %lf\n",P2);

    Y0 = ffunction(P0);
    Y1 = ffunction(P1);
    Y2 = ffunction(P2);

    for ( K = 1; K <= Max ; K++) {

        if(Satisfied) break;

        H0 = P0 - P2;            /*  Form differences  */
        H1 = P1 - P2;
        C  = Y2;
        E0 = Y0 - C;
        E1 = Y1 - C;
        Det = H1 * H0 * (H0 - H1);           /* Compute determinants  */
        A  = (E0 * H1 - H0 * E1) / Det;      /* Solve linear system   */
        B  = (H0 * H0 * E1 - H1 * H1 * E0) / Det;

        if( (B * B) > (4 * A * C) )  Disc = sqrt(B * B - 4 * A * C);
        else Disc = 0;  /* This suppresses complex roots */

        if( B < 0 ) Disc = -Disc;    /* Find the smallest root of  */
        Z = -2 * C / (B + Disc);     /* the quadratic              */
        P3 = P2 + Z;                 /* Newest approximation       */

        if ( fabs(P3 - P1) < fabs(P3 - P0) ) {
            U = P1;
            P1 = P0;
            P0 = U;
            V = Y1;
            Y1 = Y0;
            Y0 = V;
        }

        if ( fabs(P3 - P2) < fabs(P3 - P1) ) {
            U = P2;
            P2 = P1;
            P1 = U;
            V = Y2;
            Y2 = Y1;
            Y1 = V;
        }

        P2 = P3;
        Y2 = ffunction(P2);

        RelErr = 2 * fabs(Z) / ( fabs(P2) + Small );

        if ( (RelErr < Delta) && (fabs(Y2) < Epsilon) ) Satisfied = 1;

    }   /* End of for-loop */

    printf("----------------------------------------------\n");
    printf("The number of iterations is: %d\n",K-1);


    if (Satisfied) {
    printf("The computed root of f(x) is: %lf\n",P2);
    printf("Consecutive iterates are closer than %lf\n",fabs(Z));
    printf("The current function value is %lf\n",Y2);
    printf("----------------------------------------------\n");
    }

    else printf("Muller's method did not find a root of f(x) = 0 .\n");

    printf("----------------------------------------------\n");

}   /* End of main program */

