/*
---------------------------------------------------------------------------
 Algo5-3.c   C program for implementing Algorithm 5.3
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 5.3 (Non-linear Curve Fitting).
 Section   5.2, Curve Fitting, Page 280
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 5.3 (Nonlinear Curve Fitting).

 To  construct a nonlinear fit  y = g(x)  given the  N  data points 
 (x_1,y_1),...,(x_N,y_N)  by using "data linearization."
 Available curves are :

     A                D               1                 x
 y = - + B,    y = ------,    y = ---------,    y = ---------
     x              x + C           Ax + B           A + Bx

                                              A              -1
 y = A ln(x) + B,   y = C exp(Ax),   y = C * x ,   y = (Ax +B)

                                    L
 y = C * x * exp(-Dx),    y = --------------- .
                               1 + C*exp(Ax)

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  Main program for algorithm 5.3  */

    void main()

{
    int K;                     /* loop counter                   */
    int N;                     /* INPUT: Number of points        */
    double A, B;               /* Slope and Y-intersection       */
    double X[20];              /* x-coordinates of points        */
    double Y[20];              /* y-coordinates of points        */
    double Xmean;              /* Average value for x coord.     */
    double Ymean;              /* Average value for y coord.     */
    double SumX;               /* Sum of X coordinate values     */
    double SumXY;              /* Sum of product X*Y             */
    int Ncase;                 /* Case of nonlinear Regression   */
    double C, D, L;            /* Coefficients in formulas      */

    printf("--------Try examples on page 282----------------\n");
    printf("Please enter number N of data points:\n");
    printf("EXAMPLE: N = 5, so type: 5\n");
    scanf("%d",&N);
    if(N > 21)
    {
     printf(" Not prepared for more than 21 points. Terminating. Sorry\n");
     exit(0);
    }

    printf("The number of data points  N =  %d\n", N);
    printf("----------------------------------------------\n");

        printf("INPUT of data points :\n");
        printf("Example : x = -2 and y = 1, so type : -2, 1\n");
        printf(" \n");

    for(K = 1; K <= N; K++)
    {
        printf("data point number %d please.\n",K);
        scanf("%lf, %lf", &X[K-1],&Y[K-1]);
    }


    /* Select case */

    printf("=======Select one of the following cases=======\n");
    printf("(1) y = A/x + B              (2) y = D/[x+C]\n");
    printf("(3) y = 1/[Ax + B]           (4) y = x/[A + Bx]\n");
    printf("(5) y = A ln(x) + B          (6) y = C * exp(Ax)\n");
    printf("(7) y = C * x^A              (8) y = [Ax + B]^-2\n");
    printf("(9) y = C * x exp(-Dx)\n");
    printf("(10) y = L / [ 1 + C * exp(A*x) ]\n");
    printf("===============================================\n");
    do
    { 
     printf("Type the number of the case you have to solve:\n");
     scanf("%d", &Ncase);
    }while( (Ncase < 1) || (Ncase > 10) );


    /* make transformation(s) to linearize the data j = 1, 2,..., N */
        
    switch(Ncase)
    {
     case 1:  for (K = 1; K <= N; K++) X[K-1] = 1.0 / X[K-1];
     break;
      
     case 2:  for (K = 1; K <= N; K++) X[K-1] = X[K-1] * Y[K-1];
     break;

     case 3:  for (K = 1; K <= N; K++) Y[K-1] = 1.0 / Y[K-1];
     break;

     case 4: 
     for(K = 1; K <= N; K++)  X[K-1] = 1.0 / X[K-1];
     for(K = 1; K <= N; K++)  Y[K-1] = 1.0 / Y[K-1];
     break;

     case 5:
     for(K = 1; K <= N; K++) X[K-1] = log(X[K-1]);
     break;

     case 6: 
     for(K = 1; K <= N; K++) Y[K-1] = log(Y[K-1]);
     break;

     case 7:  
     for(K = 1; K <= N; K++)  X[K-1] = log( X[K-1] );
     for(K = 1; K <= N; K++)  Y[K-1] = log( Y[K-1] );
     break;

     case 8: 
     for(K = 1; K <= N; K++) Y[K-1] = sqrt(Y[K-1]);
     break;

     case 9:
     for(K = 1; K <= N; K++) Y[K-1] = log( Y[K-1] / X[K-1] );
     break;
      
     case 10:
     printf(" Please input the constant   L  :\n");
     scanf("%lf", &L);
     for(K = 1; K <= N; K++) Y[K-1] = log(L / Y[K-1]  - 1.0);
     break;

  }
        

    Xmean = 0.0;

    /* calculate X-mean value */

    for(K = 1; K <= N; K++) Xmean = Xmean + X[K-1];
    Xmean = Xmean / N;

    Ymean = 0.0;

    /* calculate Y-mean value */

    for(K = 1; K <= N; K++) Ymean = Ymean + Y[K-1];
    Ymean = Ymean / N;


    SumX = 0.0;

    /* Sum (x_k - Xmean) squared */

    for(K = 1; K <= N; K++) SumX = SumX + pow( (X[K-1] - Xmean),2 );


     SumXY = 0.0;

    /* Sum (x_k - Xmean) * (Y_k - Ymean) */

    for(K = 1; K <= N; K++) SumXY = SumXY +
                            ( (X[K-1] - Xmean) * (Y[K-1] - Ymean) );


    A = SumXY/SumX;         /* Compute the slope          */
    B = Ymean - A * Xmean;  /* Compute the y-intersection */

    /* Make coefficient transformation(s) if necessary */

    switch(Ncase)
  {
      case 2:

      C = -1.0 / A;
      D = -B / A;
      break;

      case 6: C = exp(B);
      break;

      case 7: C = exp(B);
      break;

      case 9: 
      
      C = exp(B);
      D = -A;
      break;

      case 10: C = exp(B);
      break;
      
    }
    
    /* Output */

    printf("The least-squares curve for the data is:\n");
    printf("----------------------------------------------\n");

    switch(Ncase)
    {
      case 1:  
      printf("y = %lf / x  + %lf\n", A, B);
      break;
      
      case 2:  
      printf("y = %lf / [ x + %lf]\n", D, C);
      break;

      case 3:  
      printf("y = 1 / [ %lf * x + %lf ]\n", A, B);
      break;

      case 4: 
      printf("y = x / [ %lf  + %lf * x ]\n", A, B);
      break;

      case 5:  
      printf("y = %lf * ln(x) + %lf\n", A, B);
      break;

      case 6:  
      printf("y = %lf * exp(%lf * x)\n", C, A);
      break;

      case 7:  
      printf("%lf * x^(%lf)\n", C, A);
      break;

      case 8: 
      printf("y = [ %lf * x + %lf ]^(2)\n", A, B);
      break;
 
      case 9:
      printf("y = %lf * x * exp(- %lf * x)\n", C, D); 
      break;

      case 10:
      printf("y = %lf / [ 1 + %lf * exp(%lf * x)\n", L, C, A);
      break;

  }
 

}   /* End of main program */

