/*
---------------------------------------------------------------------------
 Algo5-4.c   C program for implementing Algorithm 5.4
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 5.4 (Cubic Splines).
 Section   5.3, Interpolation by Spline Functions, Page 297
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 5.4 (Cubic Splines).

 To  construct and evaluate a cubic spline interpolant  S(x)  for
 (N + 1)  data points  (x_0,y_0), (x_1,y_1), ... , (x_N,y_N).
 Provision is made for the following choices of endpoint constraints;

 (i)   "Clamped cubic spline"; specify S'(x_0) and S'(x_N).

 (ii)  "Natural cubic spline" (a "relaxed curve").

 (iii) Extrapolate S''(x) to the endpoints.

 (iv)  S''(x) is constant near the endpoints.

 (v)   Specify S''(x) at each endpoints.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


#define MAX 20

/*  Main program for algorithm 5.4  */

    void main()

{
    int K, J;                  /* Loop counter                   */
    int N;                     /* INPUT: Number of points -1     */
    double X[MAX];             /* x-coordinates of points        */
    double Y[MAX];             /* y-coordinates of points        */
    double H[MAX];             /* Differences in abscissa        */       
    double D[MAX];             /* Difference quotients           */
    double C[MAX];             /* Superdiagonal elements         */
    double A[MAX];             /* Subdiagonal elements           */
    double B[MAX];             /* Diagonal elements              */
    double V[MAX];             /* Column vector                  */
    double M[MAX];             /*                                */
    double SD, SDD;            /* S'(x_0) and S''(x_N)            */
    double SDN, SDDN;          /* S'(x_N) and S''(x_N)            */
    double T;                  /*                                */
    double S[MAX][4];          /* Coefficients for polynomials   */
    double Z;                  /* Value of spline S(x)           */    
    int Ncase;                 /* Choice of condition            */
    double W;                  
    int q;                     /* For comparison to end input    */
    double x;                  /* Independent variable           */  


    printf("--Try examples in section 5.3 ----------\n");
    printf("Please enter number of data points:\n");
    printf("EXAMPLE: Five (X,Y) pairs , so type: 5\n");
    scanf("%d",&N);
    N--;                /* Input was taken as N + 1 data points */
    if(N > MAX)
    {
    printf(" Not prepared for more than %d points. Terminating. Sorry\n",MAX);
    exit(0);
    }

    printf("The number of data points  N =  %d\n", N+1);
    printf("----------------------------------------------\n");

        printf("INPUT of data points :\n");
        printf("Example : x = -2 and y = 1, so type : -2, 1\n");
        printf(" \n");

    for(K = 1; K <= N+1; K++)
    {
        printf("data point number %d please.\n",K);
        scanf("%lf, %lf", &X[K-1],&Y[K-1]);
        printf("You entered : %lf %lf\n", X[K-1], Y[K-1]);
    } 

    H[0] = X[1] - X[0];             /* Difference in abscissa */
    D[0] = ( Y[1] - Y[0] ) / H[0];  /* Difference quotient    */
    
    for(K = 1; K <= N-1; K++)
    {
      H[K] = X[K+1] - X[K];                /* Differences in abscissa */
      D[K] = ( Y[K+1] - Y[K] ) /  H[K];    /* Difference quotients    */
      A[K] = H[K];                         /* Subdiagonal elements    */
      B[K] = 2 * ( H[K-1] + H[K] );        /* Diagonal elements       */
      C[K] = H[K];                         /* Superdiagonal elements  */
    }

    /* Determine the column vector */

    for(K = 1; K <= N-1; K++) V[K] = 6.0 * ( D[K] - D[K-1] );

    /* Select case */

    printf("=======Select one of the following cases=======\n");
    printf("(1) Clamped cubic spline : with S'(x_0) and S''(x_N)\n");
    printf("(2) Natural cubic spline : ( a 'relaxed curve' )\n");
    printf("(3) Extrapolate S''(x) to the endpoints\n");
    printf("(4) S''(x) is constant near the endpoints\n");
    printf("(5) With specifying S''(x) at each endpoint\n");
    printf("===============================================\n");
    do
    { 
     printf("Type the number of the case you have to solve:\n");
     scanf("%d", &Ncase);
    }while( (Ncase < 1) || (Ncase > 5) );

    /* make transformation(s) to linearize the data j = 1, 2,..., N */
        
    switch(Ncase)
    {
     case 1:  
     /* first do INPUT */
     printf("Please enter S'(x_0) and S'(x_N)\n");
     printf("Example : 2.4, 5.66 \n");
     scanf("%lf, %lf",&SD, &SDN);
     printf("You entered S'(x_0) = %lf and S'(x_N) = %lf\n", SD, SDN);

     B[1]   -= H[0] / 2.0;
     V[1]   -= 3.0 * (D[0] - SD);
     B[N-1] -= H[N-1] / 2.0;
     V[N-1] -= 3.0 * (SDN - D[N-1]);
     break;
      
     case 2:  
     M[0] = 0.0;
     M[N] = 0.0;
     break;

     case 3:  
     B[1]    +=  H[0] + H[0] * H[0] / H[1];
     C[1]    -=  H[0] * H[0] / H[1];
     B[N-1]  +=  H[N-1] + H[N-1] * H[N-1]/H[N-2];
     A[N-2]  -=  H[N-1] * H[N-1] / H[N-2];     
     break;

     case 4: 
     B[1]    += H[0];
     B[N-1]  += H[N-1];
     break;

     case 5:
     /* first do INPUT */
     printf("Please enter S''(x_0) and S''(x_N)\n");
     printf("Example : 2.4, 5.66 \n");
     scanf("%lf, %lf",&SDD, &SDDN);
     printf("You entered S''(x_0) = %lf and S''(x_N) = %lf\n", SDD, SDDN);

     V[1]    -= H[0] * SDD;
     V[N-1]  -= H[N-1] * SDDN;
     break;

  }

    /* Gaussian elimination is used to produce an upper-triangular
       system with two-diagonals   */

    for(K = 2; K <= N-1; K++) 
    {
       T = A[K-1] / B[K-1];
       B[K] -= T * C[K-1];
       V[K] -= T * V[K-1];
    }
     
    /* Back substitution is used to find m_k */

    M[N-1] = V[N-1] / B[N-1];

    for(K = N-2; K >= 1; K--) M[K] = ( V[K] - C[K] * M[K+1] ) / B[K];

    
    
    /* Determine the values M(0) and M(N) */

    switch(Ncase)
    {
      case 1:  
      M[0] = 3.0 * (D[0] - SD) / H[0] - M[1] / 2.0;
      M[N] = 3.0 * (SDN - D[N-1]) / H[N-1] - M[N-1] / 2.0;
      break;
      
      case 2:  
      M[0] = 0;
      M[N] = 0;
      break;

      case 3:  
      M[0] = M[1] - H[0] * (M[2] - M[1]) / H[1];
      M[N] = M[N-1] + H[N-1] * (M[N-1] - M[N-2]) / H[N-2];
      break;

      case 4: 
      M[0] = M[1];
      M[N] = M[N-1];
      break;

      case 5:  
      M[0] = SDD;
      M[N] = SDDN;
      break;

    }
 
      /* Compute and store the coefficients for each cubic
         polynomial S_k(x)    */

      for (K = 0; K <= N-1; K++)
      {
        S[K][0] = Y[K];                
        S[K][1] = D[K] - H[K] * ( 2.0 * M[K] + M[K+1] ) / 6.0;
        S[K][2] = M[K] / 2.0;
        S[K][3] = ( M[K+1] - M[K] ) / ( 6.0 * H[K] );
      }

    /* Procedure for evaluating the cubic spline above on [x_0,x_N] */

    /* Input the independent variable  x */

    printf("-----------------------------------------------------\n");
    printf("Now comes a LOOP for input of the independent variable.\n");       
    printf("-----------------------------------------------------\n");

   for(; ; )
   {
    printf("TYPE  q  IF you want to STOP, else any other key.\n");
	scanf ("%c", &q);
    if( q == 'q') break ;

    printf("Ok. You want to continue, so.....\n");
    printf("Please enter the independent variable X :\n");
    scanf("%lf", &x);
    printf("You entered %lf  for   x\n", x);
    printf("-----------------------------------------------------\n");

    /* Find the interval so that  x  lies in [x_k, x_(k+1)] */
     
    for ( J = 1; J <= N; J++)
    {
      if( (X[J-1] <= x) && ( x <= X[J] ) )
      {
         K = J - 1;
         J = N;
      }
    }   

    if( x == X[0] ) K = 0;

    /* Evaluate the spline */

    W = x - X[K];
    Z = ( ( S[K][3] * W + S[K][2] ) * W + S[K][1] ) * W  + S[K][0];   

    /* Output */

    printf("The value of the spline S(x) is : %lf\n", Z);

   } /* End of for-loop for INPUT */

}   /* End of main program */

