/*
---------------------------------------------------------------------------
 Algo7-3.c   C program for implementing Algorithm 7.3
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 7.3 (Recursive Trapezoidal Rule).
 Section   7.3, Recursive Rules and Romberg Integration, Page 378
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 7.3 (Recursive Trapezoidal Rule).  To  approximate the integral

     b                    J
     /              h    2
     | f(x)dx   =   - * sum  [ f(x_(k-1)) + f(x_k) ]
     /              2   k=1
     a


 by using the trapezoidal rule and successively and increasing number
 of subintervals of [A,B]. The J-th iteration samples f(x) at
       J
      2  +  1 equally spaced points.

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#define MAX 25
#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction"   */

    double ffunction(double x)

    {
        return ( 1 / ( 1 + pow(x,2) ) );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 7.3  */

    void main()

{
    int J, K;                  /* loop counters                  */
    int M;                     /* number of subintervals         */
    int N;                     /* INPUT : number of times subintervals
                                          are bisected  */
    double A,B;                /* INPUT : boundaries of integral */
    double H;                  /* Subinterval width              */
    double SUM     = 0;        /*                                */
    double T[MAX];             /* approx. integral values        */


    printf("----------------------------------------------\n");
    printf("Please enter the boundaries of the integral [A,B]\n");
    printf("EXAMPLE: A = -1 and B = 1, so type: -1 1\n");
    scanf("%lf %lf",&A, &B);
    printf("The boundaries of the integral are: %lf %lf\n",A, B);
    printf("----------------------------------------------\n");
    printf("How often shall SUBINTERVALS be bisected ? [ <= MAX]\n");
    scanf("%d",&N);

    if( N > MAX ) {
         printf(" :-() Maximum is %d ! Start again.\n", MAX);
         exit(1);
    }
    printf("You say : %d subintervals.\n",N);

    M = 1;
    H = B - A;                                      /*  Subinterval width  */

    T[0] = H * ( ffunction(A) - ffunction(B) ) / 2; /*  Compute one
                                                        trapezoid */

    printf("The approximate value of the integral using 1 subinterval\n");
    printf("and the trapezoidal rule is %lf\n",T[0]);


    for ( J = 1; J <= N; J++ ) {
         M = 2 * M;
         H = H / 2;
         SUM = 0;

       for ( K =  1; K <= M/2; K++ ) {
         SUM = SUM + ffunction( A + H * (2 * K - 1) );
       }

         T[J] = T[J-1] / 2  + H * SUM;

    printf("----------------------------------------------\n");

    printf(" The approximate value of the integral using\n");
    printf(" %d subintervals and the trapezoidal rule is :\n", M);
    printf(" %lf\n",T[J] );
    printf(" ...computing ... computing ...\n");

    printf("----------------------------------------------\n");

    }  /* end of outer for-loop */
}   /* End of main program */

