/*
---------------------------------------------------------------------------
 Algo9-4.c   C program for implementing Algorithm 9.4
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 9.4 (Runge-Kutta Method of Order 4).
 Section   9.5, Runge-Kutta Methods, Page 460
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 9.4 (Runge-Kutta Method of Order 4).

 To approximate the solution of the initial value problem  y' = f(t,y)
 with  y(a) = y_0  over [a,b] by using the formula

                      h
 y_(k+1)  =  y_k  +  --- [K_1 + 2*K_2 + 2*K_3 + K_4]
                      6

 User has to supply  functions named : ffunction 
 An example is included in this program.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define MAX 500


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double t, double y);

/*  EXAMPLE for "f1function"   */

    double ffunction(double t, double y)

    {
        return ( (t - y) / 2.0 );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 9.4  */

    void main(void)

{
    int J;                     /* Loop counter                   */
    int M;                     /* INPUT: Number of steps         */
    double A, B, Y[MAX];       /* Endpoints and inital value     */
    double H;                  /* Compute the step size          */
    double T[MAX]; 
    double K1, K2, K3, K4;     /* Function values                */
    double t, y;


    printf("----------- Taylor's Method -------------------\n");
    printf("--------- Example 9.10 on page 454 -------------\n");
    printf("-----------------------------------------------\n");
    printf("Please enter endpoints of the interval [A,B]:\n");
    printf("For used Example type  : 0, 3\n");
    scanf("%lf, %lf", &A, &B);
    printf("You entered [%lf, %lf]\n", A, B);
    printf("-----------------------------------------\n");
    printf("Please enter number of steps: (Not more than 500 !)\n");
    scanf("%d",&M);
    if(M > MAX)
    {
    printf(" Not prepared for more than %d steps. Terminating. Sorry\n",MAX);
    exit(0);
    }

    printf("-----------------------------------------\n");
    printf("Please enter initial value Y[0] :\n");
    printf("For used Example type : 1\n");
    scanf("%lf", &Y[0]);
    printf("You entered Y[0] = %lf\n", Y[0]);

    /* Compute the step size */

    H    = (B - A) / M;      

    /* Initialize the variable */

    T[0] = A;


    for(J = 0; J <= M-1; J++)
    {
      t  = T[J];
      y  = Y[J];
      K1 = H * ffunction(t,y);  
      K2 = H * ffunction(t + H/2.0, y + 0.5 * K1);  
      K3 = H * ffunction(t + H/2.0, y + 0.5 * K2);  
      K4 = H * ffunction(t + H, y + K3);  
      Y[J+1] = y + ( K1 + 2.0 * K2 + 2.0 * K3 + K4 ) / 6.0;
      T[J+1] = A + H * (J+1);
    }

    /* Output */

    for ( J = 0; J <= M; J++)
    {
       printf("J = %d, T[J] = %lf, Y[J] = %lf\n", J, T[J], Y[J]);
    } 

}   /* End of main program */

